﻿#Custom variables
$wdTypes = Add-Type -AssemblyName 'Microsoft.Office.Interop.Word' -Passthru
$wdSaveFormat = $wdTypes | Where {$_.Name -eq "wdSaveFormat"}
$NeedIt = 0
$SignatureName = 'Signature Mail'
$SignatureVer = '1.0'
$SigSource = '\\Server\Share\Template Folder' #Provide full path to the *.docx signature file
$UseSignOnNew = '1'        #If set to '0', the signature will be added as signature for new mails.
$UseSignOnReply = '1'      #If set to '0', the signature will be added as signature for reply mails.
$ForceSignatureNew = '0'   #If set to '0', the signature will be editable in Outlook and if set to '1' will be non-editable and forced - forced also as reply.
$ForceSignatureReply = '0' #If set to '0', the signature will be editable in Outlook and if set to '1' will be non-editable and forced.
$DefaultAddress = 'Street 11'
$DefaultCity = 'My City'
$DefaultPOBox = '10000'
$DefaultTelephone = ''
 
#Environment variables
$AppData=(Get-Item env:appdata).value
$SigPath = '\Microsoft\Signatures'
$LocalSignaturePath = $AppData+$SigPath
$RemoteSignaturePathFull = $SigSource
$UserName = $env:username
$Filter = '(&(objectCategory=User)(samAccountName=$UserName))'
$Searcher = New-Object System.DirectoryServices.DirectorySearcher
$Searcher.Filter = $Filter
$ADUserPath = $Searcher.FindOne()
$ADUser = $ADUserPath.GetDirectoryEntry()
$ADDisplayName = $ADUser.DisplayName
$ADEmailAddress = $ADUser.mail
$ADTitle = $ADUser.title
$ADMobile = $ADUser.mobile
$ADStreetAddress = $ADUser.streetaddress
$ADCity = $ADUser.l
$ADPOBox = $ADUser.postofficebox
$ADCustomAttribute1 = $ADUser.extensionAttribute1
$ADOffice = $ADUser.physicalDeliveryOfficeName
$ADModify = $ADUser.whenChanged

#Retriving implementations of Signatures from registry
$SignatureInstVer = Get-ItemProperty -Name 'VersionSignature' -Path HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -ErrorAction SilentlyContinue
If (Get-ItemProperty -Name 'ADChangeDate' -Path HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -ErrorAction SilentlyContinue)
{
    $RegistryKey = Get-ItemProperty -Name 'ADChangeDate' -Path HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -ErrorAction SilentlyContinue
    $RegDate = $RegistryKey.ADChangeDate.ToDateTime($_.Value) 
}
Else { 
    $RegDate = (Get-Date).Date.AddDays(-365)
} 

If (Get-ItemProperty -Name 'VersionSignature' -Path HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -ErrorAction SilentlyContinue)
{
    $SignatureInstVer = Get-ItemProperty -Name 'VersionSignature' -Path HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -ErrorAction SilentlyContinue
    $SignatureInstVer = $SignatureInstVer.VersionSignature.ToString()
}
Else { 
    $SignatureInstVer = '0'
} 

#Using data to determinate if change of signature is needed
If ($SignatureVer -ne $SignatureInstVer)
{
    $NeedIt = 1
}
If ($ADUser.whenChanged -ge ($RegDate).Date)
{
    $wshell = New-Object -ComObject Wscript.Shell
    $Result = $wshell.Popup("Your account related data was changed. If it changed something, that affect signature, please click on OK button. Otherwise click Cancel.",0,"All Done",0x1)
    If ($Result -eq 2 -And $NeedIt -eq 0)
    {
        $RegDate = (Get-Date).Date.ToString("dd. MM. yyyy")
        Set-ItemProperty HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -Name 'ADChangeDate' -Value $RegDate
        break
    }
    $NeedIt = 2
}

#Implementation Of Signature

If ($NeedIt -gt 0)
{
#Check signature path (needs to be created if a signature has never been created for the profile
if (!(Test-Path -path $LocalSignaturePath)) {
       New-Item $LocalSignaturePath -Type Directory
}

#Copy signature templates from domain to local Signature-folder
$FullSource=$Sigsource+'\'+$SignatureName+'.docx'
Copy-Item $FullSource $LocalSignaturePath -Recurse -Force
$ReplaceAll = 2
$FindContinue = 1
$MatchCase = $False
$MatchWholeWord = $True
$MatchWildcards = $False
$MatchSoundsLike = $False
$MatchAllWordForms = $False
$Forward = $True
$Wrap = $FindContinue
$Format = $False
 
#Insert variables from Active Directory to rtf signature-file
$MSWord = New-Object -ComObject word.application
$fullPath = $LocalSignaturePath+'\'+$SignatureName+'.docx'
$MSWord.Documents.Open($fullPath)
 
#User Name
$FindText = 'DisplayName' 
$ReplaceText = $ADDisplayName.ToString() 
$MSWord.Selection.Find.Execute($FindText, $MatchCase, $MatchWholeWord,    $MatchWildcards, $MatchSoundsLike, $MatchAllWordForms, $Forward, $Wrap,      $Format, $ReplaceText, $ReplaceAll       )
$LogInfo += $NL+$ReplaceText      
 
#E-Mail Address
$FindText = 'E-Mail' 
$ReplaceText = $ADEmailAddress.ToString() 
$MSWord.Selection.Find.Execute($FindText, $MatchCase, $MatchWholeWord,    $MatchWildcards, $MatchSoundsLike, $MatchAllWordForms, $Forward, $Wrap,      $Format, $ReplaceText, $ReplaceAll       )
$LogInfo += $NL+'Email: '+$ReplaceText      
 
#Title       
$FindText = 'Title'
$ReplaceText = $ADTitle.ToString()
$MSWord.Selection.Find.Execute($FindText, $MatchCase, $MatchWholeWord,    $MatchWildcards, $MatchSoundsLike, $MatchAllWordForms, $Forward, $Wrap,      $Format, $ReplaceText, $ReplaceAll       )
$LogInfo += $NL+$ReplaceText
       
#Address
 
If ($ADStreetAddress -ne '') { 
    $ReplaceText1 = $ADStreetAddress.ToString()
}
Else {
    $ReplaceText1 = $DefaultAddress
    }

If ($ADCity -ne '') { 
    $ReplaceText2 = $ADCity.ToString()
}
Else {
    $ReplaceText2 = $DefaultCity 
    }

If ($ADPOBox -ne '') { 
    $ReplaceText3 = $ADPOBox.ToString()
}
Else {
    $ReplaceText3 = $DefaultPOBox 
    }

    $ReplaceText = $ReplaceText1 +', '+ $ReplaceText2 +', '+ $ReplaceText3

       $FindText = 'AllAddress'
       $MSWord.Selection.Find.Execute($FindText, $MatchCase, $MatchWholeWord,    $MatchWildcards, $MatchSoundsLike, $MatchAllWordForms, $Forward, $Wrap,      $Format, $ReplaceText, $ReplaceAll       )
       $LogInfo += $NL+$ReplaceText
       
#Mobile
If ($ADMobile -ne "") { 
       $FindText = 'MobileN'
       $ReplaceText = $ADMobile.ToString()
   }
Else {
       $FindText = 'MobileN'
    $ReplaceText = "".ToString()
       }
       $MSWord.Selection.Find.Execute($FindText, $MatchCase, $MatchWholeWord,    $MatchWildcards, $MatchSoundsLike, $MatchAllWordForms, $Forward, $Wrap,      $Format, $ReplaceText, $ReplaceAll       )
    $LogInfo += $NL+$ReplaceText

#Working Area
If ($ADOffice -ne "") { 
       $FindText = 'WorkingOffice'
       $ReplaceText = $ADOffice.ToString()
   }
Else {
       $FindText = 'WorkingOffice'
    $ReplaceText = "".ToString()
       }
       $MSWord.Selection.Find.Execute($FindText, $MatchCase, $MatchWholeWord,    $MatchWildcards, $MatchSoundsLike, $MatchAllWordForms, $Forward, $Wrap,      $Format, $ReplaceText, $ReplaceAll       )
    $LogInfo += $NL+$ReplaceText

#Save HTML
$path = $LocalSignaturePath+'\'+$SignatureName+'.htm'
$MSWord.ActiveDocument.saveas($path, $wdSaveFormat::wdFormatHTML);
    
#Save RTF 
$path = $LocalSignaturePath+'\'+$SignatureName+'.rtf'
$MSWord.ActiveDocument.SaveAs($path, $wdSaveFormat::wdFormatRTF);
	
#Save TXT    
$path = $LocalSignaturePath+'\'+$SignatureName+'.txt'
$MSWord.ActiveDocument.SaveAs($path, $wdSaveFormat::wdFormatRTF);
	
$MSWord.ActiveDocument.Close();
$MSWord.Quit();
 
If (Test-Path HKCU:'\Software\Microsoft\Office\15.0')
 
{
    $Outlook = 'Outlook'
    if ($Outlook -ne $null)
    {
        Stop-Process -Name $Outlook -Force
    }
 
    $MSWord = New-Object -comobject word.application
    $EmailOptions = $MSWord.EmailOptions
    $EmailSignature = $EmailOptions.EmailSignature
    $EmailSignatureEntries = $EmailSignature.EmailSignatureEntries
    If ($UseSignOnNew -eq '1')
    {
        $EmailSignature.NewMessageSignature="$SignatureName"
    }
    If ($UseSignOnReply -eq '1')
    {
        $EmailSignature.ReplyMessageSignature="$SignatureName"
    }
    Stop-Process -Name $Outlook

If ($ForceSignatureNew -eq '1')
{
    If (Get-ItemProperty -Name 'NewSignature' -Path HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -ErrorAction SilentlyContinue) { } 
    Else { 
        New-ItemProperty HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -Name 'NewSignature' -Value $SignatureName -PropertyType 'String' -Force 
        } 
}

If ($ForceSignatureReply -eq '1')
{
    If (Get-ItemProperty -Name 'ReplySignature' -Path HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -ErrorAction SilentlyContinue) { } 
    Else { 
        New-ItemProperty HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -Name 'ReplySignature' -Value $SignatureName -PropertyType 'String' -Force
        } 
}
}

#Write Signature specified Registry Values
If (Get-ItemProperty -Name 'VersionSignature' -Path HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -ErrorAction SilentlyContinue)
{
    Set-ItemProperty HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -Name 'VersionSignature' -Value $SignatureVer
}
Else { 
    New-ItemProperty HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -Name 'VersionSignature' -Value $SignatureVer -PropertyType 'String' 
} 

$RegDate = (Get-Date).Date.ToString("dd. MM. yyyy")
If (Get-ItemProperty -Name 'ADChangeDate' -Path HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -ErrorAction SilentlyContinue)
{
    Set-ItemProperty HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -Name 'ADChangeDate' -Value $RegDate
}
Else { 
    New-ItemProperty HKCU:'\Software\Microsoft\Office\15.0\Common\MailSettings' -Name 'ADChangeDate' -Value $RegDate -PropertyType 'String' 
} 


$FullFile = $LocalSignaturePath+'\'+$SignatureName+'.docx'
Remove-Item $FullFile

$wshell = New-Object -ComObject Wscript.Shell
$wshell.Popup('Script is completed! You have all Mail signatures in Outlook',0,'All Done',0x0)
}
